package com.ruoyi.business.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.ruoyi.business.domain.Apply;
import com.ruoyi.business.domain.Rebot;
import com.ruoyi.business.domain.bo.AddApplyPlanBO;
import com.ruoyi.business.domain.bo.ApplyStartBO;
import com.ruoyi.business.domain.bo.ListApplyBO;
import com.ruoyi.business.domain.bo.ListRebotBO;
import com.ruoyi.business.domain.vo.JobStartVO;
import com.ruoyi.business.domain.vo.ListRebotVO;
import com.ruoyi.business.service.IApplyService;
import com.ruoyi.business.service.IRebotService;
import com.ruoyi.business.util.YinDaoHttpUtils;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.enums.ExcTypeStatus;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.business.mapper.ApplyPlanMapper;
import com.ruoyi.business.domain.ApplyPlan;
import com.ruoyi.business.service.IApplyPlanService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;

import static cn.hutool.json.XMLTokener.entity;

/**
 * 应用执行计划管理Service业务层处理
 *
 * @author LiuChengRan
 * @date 2024-06-14
 */
@Service
public class ApplyPlanServiceImpl extends ServiceImpl<ApplyPlanMapper, ApplyPlan> implements IApplyPlanService {
    @Autowired
    private IApplyService applyService;

    @Autowired
    private IRebotService rebotService;

    /**
     * 执行计划（针对立即执行的计划）
     *
     * @return
     */
    @Override
    @Transactional
    public boolean execPlan() throws IllegalAccessException {
        ListRebotBO listRebotBO = new ListRebotBO();
        listRebotBO.setStatus("idle");
        listRebotBO.setPage(1);
        listRebotBO.setSize(100);
        // 只获取空闲的机器人
        List<ListRebotVO> listRebotVOS = YinDaoHttpUtils.listRebot(listRebotBO);
        if (listRebotVOS.isEmpty()) {
            log.error("暂无空闲机器人");
            // 刷新机器人数据
//            rebotService.syn(new Rebot());
        }
        for (ListRebotVO listRebotVO : listRebotVOS) {
            List<ApplyPlan> list = this.lambdaQuery().eq(ApplyPlan::getExcType, ExcTypeStatus.ONE.getKey()).isNull(ApplyPlan::getTaskUuid).orderByAsc(ApplyPlan::getPriority).list();
            if (list.isEmpty()) {
                log.debug("没有等待执行的计划");
                break;
            }
            ApplyStartBO applyStartBO = new ApplyStartBO();
            ApplyPlan applyPlan = list.get(0);
            applyStartBO.setRobotUuid(applyPlan.getAppId());
            applyStartBO.setAccountName(listRebotVO.getRobotClientName());
            //high 高 middle 中 low 低
            applyStartBO.setPriority("high");
            JobStartVO jobStartVO = YinDaoHttpUtils.appStart(applyStartBO);
            ApplyPlan updateApplyPlan = new ApplyPlan();
            updateApplyPlan.setId(applyPlan.getId());
            updateApplyPlan.setUpdateBy("-1");
            updateApplyPlan.setUpdateTime(new Date());
            updateApplyPlan.setTaskUuid(jobStartVO.getJobUuid());
            this.updateById(updateApplyPlan);
        }

        return false;
    }

    @Override
    public boolean save(AddApplyPlanBO addApplyPlanBO) {
        List<ApplyPlan> applyPlanList = new ArrayList<>();
        Map<String, List<Apply>> appMap = applyService.lambdaQuery().in(Apply::getAppId, addApplyPlanBO.getAppId()).list().stream().collect(Collectors.groupingBy(Apply::getAppId));
        addApplyPlanBO.getAppId().forEach(e -> {
            ApplyPlan applyPlan = new ApplyPlan();
            BeanUtils.copyBeanProp(applyPlan, addApplyPlanBO);
            Apply apply = appMap.get(e).get(0);
            applyPlan.setAppName(apply.getAppName());
            applyPlan.setAppId(apply.getAppId());
            applyPlan.setAppType(apply.getAppType());
            applyPlan.setAppTypeName(apply.getAppTypeName());
            applyPlanList.add(applyPlan);
        });

        return super.saveBatch(applyPlanList);
    }

    @Override
    public boolean update(AddApplyPlanBO addApplyPlanBO) {
        List<ApplyPlan> applyPlanList = new ArrayList<>();
        Map<String, List<Apply>> appMap = applyService.lambdaQuery().in(Apply::getAppId, addApplyPlanBO.getAppId()).eq(Apply::getDeptId, SecurityUtils.getDeptId()).list().stream().collect(Collectors.groupingBy(Apply::getAppId));
        addApplyPlanBO.getAppId().forEach(e -> {
            ApplyPlan applyPlan = new ApplyPlan();
            BeanUtils.copyBeanProp(applyPlan, addApplyPlanBO);
            Apply apply = appMap.get(e).get(0);
            applyPlan.setAppName(apply.getAppName());
            applyPlan.setAppId(apply.getAppId());
            applyPlan.setAppType(apply.getAppType());
            applyPlan.setAppTypeName(apply.getAppTypeName());
            applyPlanList.add(applyPlan);
        });

        return super.updateBatchById(applyPlanList);
    }

    /**
     * 查询应用执行计划管理列表
     *
     * @param applyPlan 应用执行计划管理
     * @return 应用执行计划管理
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<ApplyPlan> list(ApplyPlan applyPlan) {
        return baseMapper.selectApplyPlanList(applyPlan);
    }
}
